# coffeelint: disable=max_line_length
zmq = require 'zmq'
debug = require("debug")("omg-zmqsockets")
# z85 = require("z85")
_ = require "lodash"
async = require 'async'

configloader = require 'omg-configloader'

{producerPrivateKey, serverPrivateKey, serverPublicKey, producerPublicKey, clientPublicKey} = configloader.keys()

producerSockets = {}

sendSocketWhenReady = (combinedName, cb) ->
  retry = ->
    if producerSockets[combinedName] and producerSockets[combinedName].send
      cb producerSockets[combinedName]
    else
      sendSocketWhenReady combinedName, cb
  setTimeout retry, 5

class ZapHandler
  constructor: ->
    debug "creating new zap handler"

    zmqzap = require("zmq-zap")
    ZAP = zmqzap.ZAP
    CurveMechanism = zmqzap.CurveMechanism

    zap = new ZAP()

    zap.use new CurveMechanism((data, callback) ->

      # This is where you'd check to see if you want to let the socket connect.
      # The CURVE mechanism lets you authenticate based on domain and address and a pre-exchanged publickey.

      # where the publickey matches the one we have for the client
      if data.publickey is producerPublicKey or data.publickey is clientPublicKey
        debug "Auth successful"
        callback null, true
      else
        debug "Auth failed"
        callback null, false
    )

    # Setup ZeroMQ ZAP socket
    # We'll use a router so that we can handle multiple requests at once
    zapSocket = zmq.socket("router")
    zapSocket.on "message", ->

      # When we get a message, send it through to the ZAP handler
      zap.authenticate arguments, (err, response) ->
        console.error "Error:", err  if err

        # Always send the response if the handler gives us one in the callback.
        # This should be done even if there is an error so that we don't block any sockets.
        zapSocket.send response  #if response


    # The socket for the ZAP handler should be bound before creating any sockets that will use it.
    # We'll use bindSync to make sure that the bind completes before we do anything else.
    try
      zapSocket.bindSync "inproc://zeromq.zap.01"
    catch error
      debug error.message

SecureBoundSocket = (name, socketType, addressToBind, properties, options) ->
  debug "binding #{socketType} #{_.padStart name, 25, " "} to #{addressToBind}"
  socket = zmq.socket socketType
  if properties
    for key, val of properties
      socket[key] = val
  if options
    for opt, val of options
      socket.setsockopt zmq[opt], val

  socket.curve_server = 1
  socket.curve_secretkey = serverPrivateKey

  # socket.bind addressToBind, (err) ->
  #   if err
  #     debug "Error whilst trying to bind #{addressToBind}", err

  async.retry {times: 2, interval: 1000}, (cb) ->
    socket.bind addressToBind, cb
  , (err, result) ->
    # Don't know how to handle error, yet
    if err
      debug "could not bind socket #{name} #{socketType} #{addressToBind}. Tried 2 times!"
      socket.emit "bind_error"
      # throw new Error "could not bind socket #{name} #{socketType} #{addressToBind}. Tried 2 times"
    else
      debug "Bound #{socketType} socket for #{name} @#{addressToBind}"

  socket

SecureConnectedSocket = (name, socketType, addressToConnectTo) ->
  unless name and socketType and addressToConnectTo
    debug "You must specify name, type and address. You provided:", name, socketType, addressToConnectTo
    return
  socket = zmq.socket(socketType)
  socket.curve_serverkey = serverPublicKey
  socket.curve_publickey = producerPublicKey
  socket.curve_secretkey = producerPrivateKey
  socket.connect addressToConnectTo
  socket

module.exports =
  # create: create
  # connectPort: connect
  ZapHandler: ZapHandler
  SecureConnectedSocket: SecureConnectedSocket
  SecureBoundSocket: SecureBoundSocket
