# coffeelint: disable=max_line_length
configloader = require 'omg-configloader'
serverConfig = configloader.server

_ = require "lodash"
fs = require 'fs'
url = require 'url'
exec = require("child_process").exec
producerConnection = require "./producerConnection"
express = require("express.io")
authentication = require './authentication'
path = require 'path'
debug = require("debug")("omg-webui_index")

packageJson = require "../package.json"
prodManPackageJson = require "../../producermanager/package.json"

# getClientKeys = require './getClientKeys'

app = express()

useshttps = serverConfig.https
if useshttps
  debug "using https"
  httpsOptions = require("./httpsOptions")
  app.https(httpsOptions).io()
else
  app.http().io()
  debug "not using https"

writeConfig = ->
  configloader.write "server", serverConfig

#start apache if standard server was switched
if serverConfig.restartApache
  exec "service httpd start"
  delete serverConfig.restartApache
  writeConfig()

app.io.set 'log', true   # Express.io defaults this to false.
app.io.set 'log level', 2

app.use authentication


#use first available folder. During development "webApp/bin" is mostly not available
paths = [path.dirname(require.main.filename) + "/#{serverConfig.frontendDir}/"
  path.normalize(path.dirname(require.main.filename) + "/../webApp/build")
  path.normalize(path.dirname(require.main.filename) + "/webApp/build")
  path.normalize(path.dirname(require.main.filename) + "/../#{serverConfig.frontendDir}/")
  path.resolve __dirname, "../webApp/build"
  path.resolve __dirname, "../webApp/bin"
]
frontend = _.find paths, (onepath) ->
  debug "cheking if #{onepath} exists"
  fs.existsSync(onepath)

debug "using", frontend, "as frontend"
app.use express.static(frontend)

producerConnection.subSocket.on "message", ->
  msg = []
  Array.prototype.slice.call(arguments).forEach (arg) ->
    msg.push arg.toString()

  producer = msg[0]
  error = msg[1]
  status = msg[2]
  data = msg[3]
  if data
    try
      data = JSON.parse(data)
    catch e
      debug "failed to parse JSON: #{data}"

  publishDataForFrontend =
    outputMessage: data
    status: status
    error: error

  debug "Received published data from #{producer}"

  app.io.room(producer).broadcast(producer, JSON.stringify(publishDataForFrontend))

# newResonseMethodProducers = ["numberlist"]

socketsConnectedToReqrepbroker = {}

app.io.on "connection", (socket) ->
  socketsConnectedToReqrepbroker[socket.id] = new producerConnection.ReqSocket()
  thisReqSocket = socketsConnectedToReqrepbroker[socket.id]
  thisReqSocket.on "message", ->
    msg = []
    Array.prototype.slice.call(arguments).forEach (arg) ->
      msg.push arg.toString()

    # reqSocket ist Dealer! -> dealerNullMsg = msg[0]
    producer = msg[1]
    debug "received data from #{producer}"

    error = msg[2]
    status = msg[3]
    data = (if msg[4] then JSON.parse(msg[4]) else "")
    publishDataForFrontend =
      outputMessage: data
      status: status
      error: error

    socket.emit producer, JSON.stringify(publishDataForFrontend)

  console.log socket.id
  socket.on "disconnect", ->
    thisReqSocket.close()
    delete socketsConnectedToReqrepbroker[socket.id]

handleproducerrequest = (req) ->

  request = req.data
  {topic, producer, action, rMethod, args} = request


  if topic is "subscription"
    if action is "subscribe"
      req.io.join producer
      producerConnection.pushSocket.send [producer, "subscribe"]
      producerConnection.subSocket.subscribe producer
    if action is "unsubscribe"
      req.io.leave producer
      producerConnection.pushSocket.send [producer, "unsubscribe"]
      producerConnection.subSocket.subscribe producer

  if topic is "request" or rMethod is "reqrep"
    # if newResonseMethodProducers.indexOf producer > -1
    #   # debug "responding directly to #{producer}"
    #   return

    #   producerConnection.reqSocket.once "message", (data) ->
    #     msg = [];
    #     Array.prototype.slice.call(arguments).forEach (arg) ->
    #       msg.push arg.toString()
    #     req.io.respond msg[1]

    debug "sending request: #{producer}, #{action}"
    if args
      socketsConnectedToReqrepbroker[req.socket.id].send ["", producer, action, JSON.stringify(args)]
    else
      socketsConnectedToReqrepbroker[req.socket.id].send ["", producer, action]


  if topic is "command"
    if args
      producerConnection.pushSocket.send [producer, action, JSON.stringify(args)]
    else
      producerConnection.pushSocket.send [producer, action]

app.io.route "producerAction", handleproducerrequest

exportLogs = require "./zipLogs"
app.get "/export", exportLogs

# app.get "/clientKeys", getClientKeys

app.get '/ferrariMIB', (req, res) ->
  file = '/usr/share/snmp/mibs/FERRARI-ELECTRONIC-MIB.txt'
  res.download file

app.get '/licenses.html', (req, res) ->
  res.sendfile "/oss/licenses.html"

toggleStandardServer = require "./toggleServer"
app.io.route "toggleStandardServer", toggleStandardServer

app.get "/getServerPort", (req, res) ->
  res.send JSON.stringify(serverConfig.port)

app.get "/webuiVersion", (req, res) ->
  res.send packageJson.version

app.get "/producerVersion", (req, res) ->
  debug "requested producer version"
  res.send prodManPackageJson.version

app.get "/createCert", (req, res) ->
  createCert = require "./createCert.js"
  createCert()
  res.send JSON.stringify("done")
  res.end()

  exec "service omgwebui restart", (error, stdout, stderr) ->
    console.log error if error
    console.log stdout if stdout
    console.log stderr if stderr

app.get "/toggleHttps", (req, res)->
  serverConfig.https = !useshttps
  writeConfig()
  res.end()

  exec "service omgwebui restart", (error, stdout, stderr) ->
    console.log error if error
    console.log stdout if stdout
    console.log stderr if stderr


if useshttps
  app.listen 443 # serverConfig.port

#redirect server if https is used
  http = require("http")
  http.createServer((req, res) ->
    completehost = "http://#{req.headers["host"]}/#{req.url}"
    hostname = url.parse completehost
    hostname = hostname.hostname
    newLocation = "https://#{hostname}#{req.url}"
    console.log newLocation
    res.writeHead 301,
      Location: newLocation
    res.end()
    return
  ).listen serverConfig.port

else
  app.listen serverConfig.port
  debug "app should be listening on port #{serverConfig.port}"

timeoutfunction = ->
  exec "touch /tmp/*"
setTimeout timeoutfunction, 5000


checkRooms = ->
  debug "checking rooms"
  rooms = app.io.sockets.adapter.rooms
  _.each rooms, (sockets, room) ->
    _.each sockets, (enabled, socketId) ->
      #don't subscribe to rooms that are sockets
      if socketId isnt room
        debug "resubscribing to #{room}"
        producerConnection.pushSocket.send [room, "subscribe"]


setInterval checkRooms, 5000
